"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var react_1 = tslib_1.__importDefault(require("react"));
var withTheme_1 = require("../../utils/withTheme");
var Menu_1 = require("../Menu/Menu");
var Input_1 = require("../input/Input");
var Popover_1 = require("../Popover/Popover");
var Pure_1 = require("../Pure/Pure");
var Control_1 = require("../Control/Control");
var AutocompleteMenuItem_1 = require("./AutocompleteMenuItem");
var with_defaults_1 = require("../../utils/with-defaults");
exports.AUTOCOMPLETE = Symbol('Autocomplete');
var RawAutocomplete = /** @class */ (function (_super) {
    tslib_1.__extends(RawAutocomplete, _super);
    function RawAutocomplete() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = {
            isOpened: false,
        };
        _this.isFocused = false;
        _this.onInputKeyDown = function (e) {
            //keyDown may be fired from menu, don't call props handler
            if ((e.keyCode || e.which) === Control_1.KeyCode.Tab) {
                //loose focus on tab press
                _this.isFocused = false;
                _this.setState({
                    isOpened: false,
                });
            }
        };
        _this.onInputKeyPress = function (e) {
            if ((e.keyCode || e.which) === Control_1.KeyCode.Enter) {
                _this.isFocused = false;
                _this.setState({
                    isOpened: false,
                });
            }
        };
        _this.onInputChange = function (value) {
            var onValueChange = _this.props.onValueChange;
            _this.setState({
                isOpened: value && value.length !== 0,
            });
            onValueChange && onValueChange(value);
        };
        _this.onPopoverRequestClose = function () {
            _this.setState({
                isOpened: false,
            });
        };
        _this.onMenuItemSelect = function (value) {
            var onValueChange = _this.props.onValueChange;
            _this.setState({
                isOpened: false,
            });
            onValueChange && onValueChange(value);
        };
        return _this;
    }
    RawAutocomplete.prototype.render = function () {
        var _this = this;
        var _a = this.props, theme = _a.theme, Input = _a.Input, MenuItem = _a.MenuItem, Menu = _a.Menu, Popover = _a.Popover, data = _a.data, value = _a.value, filter = _a.filter, inputProps = tslib_1.__rest(_a, ["theme", "Input", "MenuItem", "Menu", "Popover", "data", "value", "filter"]);
        return (react_1.default.createElement("div", { className: theme.container },
            react_1.default.createElement(Input, tslib_1.__assign({}, inputProps, { theme: theme.Input, value: value, ref: function (el) { return (_this.input = el); }, onKeyDown: this.onInputKeyDown, onValueChange: this.onInputChange, onKeyPress: this.onInputKeyPress })),
            react_1.default.createElement(Popover, { isOpened: this.state.isOpened, theme: theme.Popover, anchor: this.input, onRequestClose: this.onPopoverRequestClose, closeOnClickAway: true },
                react_1.default.createElement(Pure_1.Pure, { data: data, value: value, Menu: Menu, filter: filter }, function () {
                    var filtered = data.filter(filter(value));
                    return (filtered.length > 0 && (react_1.default.createElement(Menu, { onItemSelect: _this.onMenuItemSelect, theme: theme.Menu }, filtered.map(function (item, i) { return (react_1.default.createElement(MenuItem, { key: i, theme: theme.MenuItem, search: value, value: item }, item)); }))));
                }))));
    };
    return RawAutocomplete;
}(react_1.default.Component));
exports.Autocomplete = withTheme_1.withTheme(exports.AUTOCOMPLETE)(with_defaults_1.withDefaults({
    Input: Input_1.Input,
    Menu: Menu_1.Menu,
    Popover: Popover_1.Popover,
    MenuItem: AutocompleteMenuItem_1.AutocompleteMenuItem,
})(RawAutocomplete));
//# sourceMappingURL=Autocomplete.js.map