"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var react_1 = tslib_1.__importDefault(require("react"));
var pure_1 = require("../../utils/pure");
var fb_1 = require("@devexperts/utils/dist/object/fb");
var ON_MARKER = 'on';
var CAPTURE_MARKER = 'Capture';
var EventListener = /** @class */ (function (_super) {
    tslib_1.__extends(EventListener, _super);
    function EventListener() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    EventListener.prototype.componentDidMount = function () {
        this.addListeners();
    };
    EventListener.prototype.componentWillUpdate = function (nextProps) {
        if (shouldResetListeners(this.props, nextProps)) {
            this.removeListeners();
        }
    };
    EventListener.prototype.componentDidUpdate = function (prevProps) {
        if (shouldResetListeners(prevProps, this.props)) {
            this.addListeners();
        }
    };
    EventListener.prototype.componentWillUnmount = function () {
        this.removeListeners();
    };
    EventListener.prototype.render = function () {
        return this.props.children;
    };
    EventListener.prototype.addListeners = function () {
        var target = this.getTarget();
        var handlers = this.getHandlers();
        Object.keys(handlers).forEach(function (key) {
            var capture = key.endsWith(CAPTURE_MARKER);
            var handler = handlers[key];
            var eventName = getEventName(key, capture);
            target.addEventListener(eventName, handler, capture);
        });
    };
    EventListener.prototype.removeListeners = function () {
        var target = this.getTarget();
        var handlers = this.getHandlers();
        Object.keys(handlers).forEach(function (key) {
            var capture = key.endsWith(CAPTURE_MARKER);
            var handler = handlers[key];
            var eventName = getEventName(key, capture);
            target.removeEventListener(eventName, handler, capture);
        });
    };
    EventListener.prototype.getTarget = function () {
        var target = this.props.target;
        if (typeof target === 'string') {
            return window[target];
        }
        return target;
    };
    EventListener.prototype.getHandlers = function () {
        // noinspection JSUnusedLocalSymbols
        var _a = this.props, target = _a.target, children = _a.children, props = tslib_1.__rest(_a, ["target", "children"]);
        var propKeys = Object.keys(props);
        var eventKeys = propKeys.filter(function (key) { return key.startsWith('on'); });
        var handlers = eventKeys.reduce(function (acc, key) {
            acc[key] = props[key];
            return acc;
        }, {});
        return handlers;
    };
    EventListener = tslib_1.__decorate([
        pure_1.PURE
    ], EventListener);
    return EventListener;
}(react_1.default.Component));
exports.EventListener = EventListener;
function getEventName(rawEventName, capture) {
    var trimmedLeft = rawEventName.startsWith(ON_MARKER) ? rawEventName.substring(ON_MARKER.length) : rawEventName;
    var trimmed = capture ? trimmedLeft.substring(0, trimmedLeft.length - CAPTURE_MARKER.length) : trimmedLeft;
    return trimmed.toLowerCase();
}
function shouldResetListeners(prevProps, nextProps) {
    var prevChildren = prevProps.children, oldProps = tslib_1.__rest(prevProps, ["children"]);
    var nextChildren = nextProps.children, newProps = tslib_1.__rest(nextProps, ["children"]);
    return !fb_1.shallowEqual(oldProps, newProps);
}
//# sourceMappingURL=EventListener.js.map