"use strict";
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var react_1 = tslib_1.__importDefault(require("react"));
var react_dom_1 = tslib_1.__importDefault(require("react-dom"));
var prefix_1 = tslib_1.__importDefault(require("@devexperts/utils/dist/dom/prefix"));
var pure_1 = require("../../utils/pure");
var Pure_1 = require("../Pure/Pure");
var Table_1 = require("../Table/Table");
var Emitter_1 = tslib_1.__importDefault(require("@devexperts/utils/dist/emitter/Emitter"));
var Scrollable_1 = require("../Scrollable/Scrollable");
var classnames_1 = tslib_1.__importDefault(require("classnames"));
var withTheme_1 = require("../../utils/withTheme");
var function_1 = require("fp-ts/lib/function");
exports.GRID = Symbol('Grid');
var EVENT_GRID = {
    BODY_SCROLL: 'EVENT_GRID:BODY_SCROLL',
    BODY_SCROLLBAR_APPEAR: 'EVENT_GRID:BODU_SCROLLBAR_APPER',
    CELL_MOUNT: 'EVENT_GRID:CELL_MOUNT',
    CELL_UPDATE: 'EVENT_GRID:CELL_UPDATE',
    GRID_MOUNT: 'EVENT_GRID:GRID_MOUNT',
    GRID_UPDATE: 'EVENT_GRID:GRID_UPDATE',
};
var GridInternalEmitter = /** @class */ (function (_super) {
    tslib_1.__extends(GridInternalEmitter, _super);
    function GridInternalEmitter() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    GridInternalEmitter.prototype.emit = function (event) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        this._emit.apply(this, tslib_1.__spreadArrays([event], args));
    };
    return GridInternalEmitter;
}(Emitter_1.default));
var GRID_CONTEXT_EMITTER = '__GRID_CONTEXT_EMITTER__';
var CONTEXT_TYPES = (_a = {},
    _a[GRID_CONTEXT_EMITTER] = function_1.constNull,
    _a);
var RawGrid = /** @class */ (function (_super) {
    tslib_1.__extends(RawGrid, _super);
    function RawGrid() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._rows = {
            head: {},
            body: {},
        };
        _this._maxColumnWidths = {};
        _this.onCellMount = function (rowIndex, columnIndex, width, isInHead) {
            var rowStorage = isInHead ? _this._rows.head : _this._rows.body;
            //set or update row storage
            if (!rowStorage[rowIndex]) {
                rowStorage[rowIndex] = {
                    columns: {},
                };
            }
            rowStorage[rowIndex].columns[columnIndex] = width;
            //detect max width
            var maxColumnWidthByIndex = _this._maxColumnWidths[columnIndex];
            if (!maxColumnWidthByIndex || (maxColumnWidthByIndex && maxColumnWidthByIndex < width)) {
                _this._maxColumnWidths[columnIndex] = width;
            }
        };
        _this.onCellRemount = function () {
            throw new Error('Grid does not support dynamic row/cell mounts');
        };
        _this.onCellUpdate = function (rowIndex, columnIndex, newWidth, isInHead) {
            //update row storage
            var rowStorage = isInHead ? _this._rows.head : _this._rows.body;
            rowStorage[rowIndex].columns[columnIndex] = newWidth;
            //update max width
            _this._maxColumnWidths[columnIndex] = Math.max.apply(Math, tslib_1.__spreadArrays(Object.keys(_this._rows.head).map(function (key) { return _this._rows.head[key].columns[columnIndex]; }), Object.keys(_this._rows.body).map(function (key) { return _this._rows.body[key].columns[columnIndex]; })));
        };
        return _this;
    }
    RawGrid.prototype.getChildContext = function () {
        var _a;
        return _a = {},
            _a[GRID_CONTEXT_EMITTER] = this._emitter,
            _a;
    };
    RawGrid.prototype.componentDidMount = function () {
        this._emitter.emit(EVENT_GRID.GRID_MOUNT, this._maxColumnWidths);
        this._emitter.off(EVENT_GRID.CELL_MOUNT, this.onCellMount);
        this._emitter.on(EVENT_GRID.CELL_MOUNT, this.onCellRemount);
        //start listening to cell updates
        this._emitter.on(EVENT_GRID.CELL_UPDATE, this.onCellUpdate);
    };
    RawGrid.prototype.componentWillMount = function () {
        this._emitter = new GridInternalEmitter();
        this._emitter.on(EVENT_GRID.CELL_MOUNT, this.onCellMount);
    };
    RawGrid.prototype.componentWillUnmount = function () {
        this._emitter.off(EVENT_GRID.CELL_MOUNT, this.onCellRemount);
        this._emitter.off(EVENT_GRID.CELL_UPDATE, this.onCellUpdate);
    };
    RawGrid.prototype.componentDidUpdate = function () {
        //this is called after all cells updated
        //notify head
        this._emitter.emit(EVENT_GRID.GRID_UPDATE, this._maxColumnWidths);
    };
    RawGrid.prototype.render = function () {
        var _a = this.props, theme = _a.theme, children = _a.children;
        return react_1.default.createElement("div", { className: theme.container }, children);
    };
    RawGrid.childContextTypes = CONTEXT_TYPES;
    RawGrid = tslib_1.__decorate([
        pure_1.PURE
    ], RawGrid);
    return RawGrid;
}(react_1.default.Component));
var RawGridHead = /** @class */ (function (_super) {
    tslib_1.__extends(RawGridHead, _super);
    function RawGridHead() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = {
            scrollLeft: 0,
            withVerticalScrollbar: false,
            columns: {},
        };
        _this.onGridBodyScroll = function (scrollLeft, scrollTop) {
            _this.setState({
                scrollLeft: scrollLeft,
            });
        };
        _this.onGridBodyScrollbarAppear = function (withHorizontalScrollbar, withVerticalScrollbar) {
            _this.setState({
                withHorizontalScrollbar: withHorizontalScrollbar,
                withVerticalScrollbar: withVerticalScrollbar,
            });
        };
        _this.onGridMount = function (columns) {
            _this.setState({
                columns: tslib_1.__assign({}, columns),
            });
        };
        return _this;
    }
    RawGridHead.prototype.componentDidMount = function () {
        var emitter = this.context[GRID_CONTEXT_EMITTER];
        emitter.on(EVENT_GRID.BODY_SCROLL, this.onGridBodyScroll);
        emitter.on(EVENT_GRID.BODY_SCROLLBAR_APPEAR, this.onGridBodyScrollbarAppear);
    };
    RawGridHead.prototype.componentWillUnmount = function () {
        var emitter = this.context[GRID_CONTEXT_EMITTER];
        emitter.off(EVENT_GRID.BODY_SCROLL, this.onGridBodyScroll);
        emitter.off(EVENT_GRID.BODY_SCROLLBAR_APPEAR, this.onGridBodyScrollbarAppear);
    };
    RawGridHead.prototype.render = function () {
        var _a;
        var _b = this.props, _c = _b.Table, Table = _c === void 0 ? Table_1.Table : _c, theme = _b.theme, _d = _b.TableHead, TableHead = _d === void 0 ? Table_1.TableHead : _d, props = tslib_1.__rest(_b, ["Table", "theme", "TableHead"]);
        var _e = this.state, scrollLeft = _e.scrollLeft, withVerticalScrollbar = _e.withVerticalScrollbar;
        var style;
        if (typeof scrollLeft !== 'undefined') {
            style = prefix_1.default({
                transform: "translateX(-" + scrollLeft + "px)",
            });
        }
        var className = classnames_1.default(theme.gridHead, (_a = {},
            _a[theme.gridHead_paddedForScrollbar] = withVerticalScrollbar,
            _a));
        //todo support multiple rows in head
        return (react_1.default.createElement("div", { className: className },
            react_1.default.createElement("div", { className: theme.gridHead__content, style: style },
                react_1.default.createElement(Pure_1.Pure, tslib_1.__assign({}, this.props, { check: this.state.columns, check2: props.children }), function () { return (react_1.default.createElement(Table, { theme: theme },
                    react_1.default.createElement(TableHead, tslib_1.__assign({ theme: theme }, props), react_1.default.Children.only(props.children)))); }))));
    };
    RawGridHead.contextTypes = CONTEXT_TYPES;
    RawGridHead = tslib_1.__decorate([
        pure_1.PURE
    ], RawGridHead);
    return RawGridHead;
}(react_1.default.Component));
var RawGridBody = /** @class */ (function (_super) {
    tslib_1.__extends(RawGridBody, _super);
    // @themr(GRID)
    function RawGridBody() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.onScroll = function (scrollLeft, scrollTop) {
            if (_this._scrollLeft !== scrollLeft) {
                _this._scrollLeft = scrollLeft;
                _this.context[GRID_CONTEXT_EMITTER].emit(EVENT_GRID.BODY_SCROLL, scrollLeft, scrollTop);
            }
        };
        _this.onUpdate = function (withHorizontalScrollbar, withVerticalScrollbar) {
            if (_this._withHorizontalScrollbar !== withHorizontalScrollbar ||
                _this._withVerticalScrollbar !== withVerticalScrollbar) {
                _this._withVerticalScrollbar = withVerticalScrollbar;
                _this._withHorizontalScrollbar = withHorizontalScrollbar;
                _this.context[GRID_CONTEXT_EMITTER].emit(EVENT_GRID.BODY_SCROLLBAR_APPEAR, withHorizontalScrollbar, withVerticalScrollbar);
            }
        };
        return _this;
    }
    RawGridBody.prototype.render = function () {
        var _a = this.props, _b = _a.Table, Table = _b === void 0 ? Table_1.Table : _b, _c = _a.TableBody, TableBody = _c === void 0 ? Table_1.TableBody : _c, theme = _a.theme, props = tslib_1.__rest(_a, ["Table", "TableBody", "theme"]);
        var scrollableTheme = {
            horizontal_scrollbar__bar: theme.horizontal_scrollbar__bar,
            vertical_scrollbar__bar: theme.vertical_scrollbar__bar,
            container: undefined,
        };
        return (react_1.default.createElement(Scrollable_1.Scrollable, { onScroll: this.onScroll, onUpdate: this.onUpdate, theme: scrollableTheme },
            react_1.default.createElement("div", { className: theme.gridBody },
                react_1.default.createElement(Table, { theme: theme },
                    react_1.default.createElement(TableBody, tslib_1.__assign({ theme: theme }, props), react_1.default.Children.map(props.children, function (child, i) {
                        return react_1.default.cloneElement(child, {
                            // dont forget
                            gridRowIndexKey: i,
                        });
                    }))))));
    };
    RawGridBody.contextTypes = CONTEXT_TYPES;
    RawGridBody = tslib_1.__decorate([
        pure_1.PURE
        // @themr(GRID)
    ], RawGridBody);
    return RawGridBody;
}(react_1.default.Component));
var RawGridRow = /** @class */ (function (_super) {
    tslib_1.__extends(RawGridRow, _super);
    // @themr(GRID)
    function RawGridRow() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = {
            columns: {},
        };
        _this.onGridMount = function (columns) {
            _this.setState({
                columns: tslib_1.__assign({}, columns),
            });
        };
        _this.onGridUpdate = function (columns) {
            _this.setState({
                columns: tslib_1.__assign({}, columns),
            });
        };
        return _this;
    }
    RawGridRow.prototype.componentWillMount = function () {
        var emitter = this.context[GRID_CONTEXT_EMITTER];
        emitter.on(EVENT_GRID.GRID_MOUNT, this.onGridMount);
        emitter.on(EVENT_GRID.GRID_UPDATE, this.onGridUpdate);
    };
    RawGridRow.prototype.componentWillUnmount = function () {
        var emitter = this.context[GRID_CONTEXT_EMITTER];
        emitter.on(EVENT_GRID.GRID_MOUNT, this.onGridMount);
        emitter.on(EVENT_GRID.GRID_UPDATE, this.onGridUpdate);
    };
    RawGridRow.prototype.render = function () {
        var _this = this;
        var _a = this.props, _b = _a.TableRow, TableRow = _b === void 0 ? Table_1.TableRow : _b, props = tslib_1.__rest(_a, ["TableRow"]);
        var rowIndex = this.props.gridRowIndexKey;
        return (react_1.default.createElement(TableRow, tslib_1.__assign({}, props), react_1.default.Children.map(this.props.children, function (child, i) {
            if (!react_1.default.isValidElement(child)) {
                return child;
            }
            var newProps = {
                gridRowIndexKey: rowIndex,
                gridColumnIndexKey: i,
            };
            if (_this.state.columns) {
                newProps.gridColumnWidth = _this.state.columns[i];
            }
            return react_1.default.cloneElement(child, newProps); //dont forget
        })));
    };
    RawGridRow.contextTypes = CONTEXT_TYPES;
    RawGridRow = tslib_1.__decorate([
        pure_1.PURE
        // @themr(GRID)
    ], RawGridRow);
    return RawGridRow;
}(react_1.default.Component));
/**
 * @enum
 */
var GridCellAlignment;
(function (GridCellAlignment) {
    GridCellAlignment["LEFT"] = "GRID_CELL_ALIGN:LEFT";
    GridCellAlignment["CENTER"] = "GRID_CELL_ALIGN:CENTER";
    GridCellAlignment["RIGHT"] = "GRID_CELL_ALIGN:RIGHT";
})(GridCellAlignment = exports.GridCellAlignment || (exports.GridCellAlignment = {}));
var RawGridCell = /** @class */ (function (_super) {
    tslib_1.__extends(RawGridCell, _super);
    // @themr(GRID)
    function RawGridCell() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    RawGridCell.prototype.componentDidMount = function () {
        var contentDOMNode = react_dom_1.default.findDOMNode(this._content);
        if (!contentDOMNode || contentDOMNode instanceof Text) {
            return;
        }
        this._width = contentDOMNode.clientWidth;
        var emitter = this.context[GRID_CONTEXT_EMITTER];
        emitter.emit(EVENT_GRID.CELL_MOUNT, this.props.gridRowIndexKey, this.props.gridColumnIndexKey, this._width, this.props.isInHead);
    };
    RawGridCell.prototype.componentDidUpdate = function () {
        var contentDOMNode = react_dom_1.default.findDOMNode(this._content);
        if (!contentDOMNode || contentDOMNode instanceof Text) {
            return;
        }
        var newWidth = contentDOMNode.clientWidth;
        if (newWidth !== this._width) {
            this._width = newWidth;
            var emitter = this.context[GRID_CONTEXT_EMITTER];
            emitter.emit(EVENT_GRID.CELL_UPDATE, this.props.gridRowIndexKey, this.props.gridColumnIndexKey, newWidth, this.props.isInHead);
        }
    };
    RawGridCell.prototype.render = function () {
        var _a, _b;
        var _this = this;
        var _c = this.props, _d = _c.TableCell, TableCell = _d === void 0 ? Table_1.TableCell : _d, _e = _c.align, align = _e === void 0 ? GridCellAlignment.CENTER : _e, gridRowIndexKey = _c.gridRowIndexKey, gridColumnIndexKey = _c.gridColumnIndexKey, gridColumnWidth = _c.gridColumnWidth, colSpan = _c.colSpan, rowSpan = _c.rowSpan, restProps = tslib_1.__rest(_c, ["TableCell", "align", "gridRowIndexKey", "gridColumnIndexKey", "gridColumnWidth", "colSpan", "rowSpan"]);
        var style;
        if (typeof gridColumnWidth !== 'undefined') {
            style = {
                width: gridColumnWidth + "px",
            };
        }
        var contentClassName = classnames_1.default(restProps.theme.gridCell__content, (_a = {},
            _a[restProps.theme.gridCell__content_left] = align === GridCellAlignment.LEFT,
            _a[restProps.theme.gridCell__content_center] = align === GridCellAlignment.CENTER,
            _a[restProps.theme.gridCell__content_right] = align === GridCellAlignment.RIGHT,
            _a));
        var tableCellTheme = tslib_1.__assign(tslib_1.__assign({}, restProps.theme), { cell: classnames_1.default(restProps.theme.cell, (_b = {},
                _b[restProps.theme.cell_left] = align === GridCellAlignment.LEFT,
                _b[restProps.theme.cell_center] = align === GridCellAlignment.CENTER,
                _b[restProps.theme.cell_right] = align === GridCellAlignment.RIGHT,
                _b)) });
        return (react_1.default.createElement(TableCell, tslib_1.__assign({}, restProps, { theme: tableCellTheme }),
            react_1.default.createElement("span", { style: style, className: restProps.theme.gridCell__placeholder },
                react_1.default.createElement("span", { className: contentClassName, ref: function (el) { return (_this._content = el); } }, restProps.children))));
    };
    RawGridCell.contextTypes = CONTEXT_TYPES;
    RawGridCell = tslib_1.__decorate([
        pure_1.PURE
        // @themr(GRID)
    ], RawGridCell);
    return RawGridCell;
}(react_1.default.Component));
exports.Grid = withTheme_1.withTheme(exports.GRID)(RawGrid);
exports.GridBody = withTheme_1.withTheme(exports.GRID)(RawGridBody);
exports.GridHead = withTheme_1.withTheme(exports.GRID)(RawGridHead);
exports.GridRow = withTheme_1.withTheme(exports.GRID)(RawGridRow);
exports.GridCell = withTheme_1.withTheme(exports.GRID)(RawGridCell);
//# sourceMappingURL=Grid.js.map