"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var react_1 = tslib_1.__importDefault(require("react"));
var Demo_1 = tslib_1.__importDefault(require("../../demo/Demo"));
var pure_1 = require("../../../utils/pure");
var Button_1 = require("../../Button/Button");
var Popover_1 = require("../Popover");
var Popover_model_1 = require("../Popover.model");
var Scrollable_1 = require("../../Scrollable/Scrollable");
var Popover_page_styl_1 = tslib_1.__importDefault(require("./theme/Popover.page.styl"));
var Control_1 = require("../../Control/Control");
var Selectbox_1 = require("../../Selectbox/Selectbox");
var Menu_1 = require("../../Menu/Menu");
var classnames_1 = tslib_1.__importDefault(require("classnames"));
var Stateful = Control_1.stateful()(Selectbox_1.Selectbox);
var StatefulOpened = Control_1.stateful('isOpened', 'onToggle', 'defaultIsOpened')(Stateful);
var scrollableTheme = {
    container: Popover_page_styl_1.default.scrollable,
};
var VericalButtonPosition;
(function (VericalButtonPosition) {
    VericalButtonPosition[VericalButtonPosition["Top"] = 0] = "Top";
    VericalButtonPosition[VericalButtonPosition["Middle"] = 1] = "Middle";
    VericalButtonPosition[VericalButtonPosition["Bottom"] = 2] = "Bottom";
})(VericalButtonPosition || (VericalButtonPosition = {}));
var HorizontalButtonPosition;
(function (HorizontalButtonPosition) {
    HorizontalButtonPosition[HorizontalButtonPosition["Left"] = 0] = "Left";
    HorizontalButtonPosition[HorizontalButtonPosition["Center"] = 1] = "Center";
    HorizontalButtonPosition[HorizontalButtonPosition["Right"] = 2] = "Right";
})(HorizontalButtonPosition || (HorizontalButtonPosition = {}));
var FixedPopoverPage = /** @class */ (function (_super) {
    tslib_1.__extends(FixedPopoverPage, _super);
    function FixedPopoverPage() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = {
            isOpened: false,
            placement: Popover_model_1.PopoverPlacement.Bottom,
            align: Popover_model_1.PopoverAlign.Left,
            horizontalButtonPosition: HorizontalButtonPosition.Center,
            verticalButtonPosition: VericalButtonPosition.Middle,
        };
        _this.onToggle = function () {
            _this.setState({
                isOpened: !_this.state.isOpened,
            });
        };
        _this.onPopoverRequestClose = function () {
            _this.setState({ isOpened: false });
        };
        _this.onPlacementSelect = function (placement) {
            var placementWasVertical = [Popover_model_1.PopoverPlacement.Top, Popover_model_1.PopoverPlacement.Bottom].includes(_this.state.placement);
            var placementWillBeVertical = [Popover_model_1.PopoverPlacement.Top, Popover_model_1.PopoverPlacement.Bottom].includes(placement);
            if (placementWasVertical && !placementWillBeVertical) {
                //placement orientation changed from vertical to horizontal
                //choose default vertical align
                _this.setState({
                    align: Popover_model_1.PopoverAlign.Top,
                });
            }
            else if (!placementWasVertical && placementWillBeVertical) {
                //placement orientation changed from horizontal to vertical
                //choose default horizontal align
                _this.setState({
                    align: Popover_model_1.PopoverAlign.Left,
                });
            }
            //finally set placement
            _this.setState({
                placement: placement,
            });
        };
        _this.onAlignSelect = function (align) {
            _this.setState({
                align: align,
            });
        };
        _this.onHorizontalAlignSelect = function (horizontalButtonPosition) {
            _this.setState({
                horizontalButtonPosition: horizontalButtonPosition,
            });
        };
        _this.onVerticalAlignSelect = function (verticalButtonPosition) {
            _this.setState({
                verticalButtonPosition: verticalButtonPosition,
            });
        };
        return _this;
    }
    FixedPopoverPage.prototype.render = function () {
        var _a;
        var _this = this;
        var _b = this.state, isOpened = _b.isOpened, placement = _b.placement, align = _b.align, horizontalButtonPosition = _b.horizontalButtonPosition, verticalButtonPosition = _b.verticalButtonPosition;
        var wrapperClassName = classnames_1.default(Popover_page_styl_1.default.wrapper, (_a = {},
            _a[Popover_page_styl_1.default.wrapperLeft] = horizontalButtonPosition === HorizontalButtonPosition.Left,
            _a[Popover_page_styl_1.default.wrapperCenter] = horizontalButtonPosition === HorizontalButtonPosition.Center,
            _a[Popover_page_styl_1.default.wrapperRight] = horizontalButtonPosition === HorizontalButtonPosition.Right,
            _a[Popover_page_styl_1.default.wrapperTop] = verticalButtonPosition === VericalButtonPosition.Top,
            _a[Popover_page_styl_1.default.wrapperMiddle] = verticalButtonPosition === VericalButtonPosition.Middle,
            _a[Popover_page_styl_1.default.wrapperBottom] = verticalButtonPosition === VericalButtonPosition.Bottom,
            _a));
        return (react_1.default.createElement(Demo_1.default, null,
            react_1.default.createElement("div", null,
                react_1.default.createElement("div", { className: Popover_page_styl_1.default.controls },
                    react_1.default.createElement("div", { className: Popover_page_styl_1.default.popoverControls },
                        react_1.default.createElement("label", { className: Popover_page_styl_1.default.label }, "Placement"),
                        react_1.default.createElement(StatefulOpened, { defaultIsOpened: false, defaultValue: Popover_model_1.PopoverPlacement.Bottom, onValueChange: this.onPlacementSelect },
                            react_1.default.createElement(Menu_1.MenuItem, { value: Popover_model_1.PopoverPlacement.Top }, "Top"),
                            react_1.default.createElement(Menu_1.MenuItem, { value: Popover_model_1.PopoverPlacement.Bottom }, "Bottom"),
                            react_1.default.createElement(Menu_1.MenuItem, { value: Popover_model_1.PopoverPlacement.Left }, "Left"),
                            react_1.default.createElement(Menu_1.MenuItem, { value: Popover_model_1.PopoverPlacement.Right }, "Right")),
                        react_1.default.createElement("label", { className: Popover_page_styl_1.default.label }, "Align"),
                        (placement === Popover_model_1.PopoverPlacement.Top || placement === Popover_model_1.PopoverPlacement.Bottom) && (react_1.default.createElement(StatefulOpened, { defaultIsOpened: false, defaultValue: Popover_model_1.PopoverAlign.Left, onValueChange: this.onAlignSelect },
                            react_1.default.createElement(Menu_1.MenuItem, { value: Popover_model_1.PopoverAlign.Left }, "Left"),
                            react_1.default.createElement(Menu_1.MenuItem, { value: Popover_model_1.PopoverAlign.Center }, "Center"),
                            react_1.default.createElement(Menu_1.MenuItem, { value: Popover_model_1.PopoverAlign.Right }, "Right"))),
                        (placement === Popover_model_1.PopoverPlacement.Left || placement === Popover_model_1.PopoverPlacement.Right) && (react_1.default.createElement(StatefulOpened, { defaultIsOpened: false, defaultValue: Popover_model_1.PopoverAlign.Top, onValueChange: this.onAlignSelect },
                            react_1.default.createElement(Menu_1.MenuItem, { value: Popover_model_1.PopoverAlign.Top }, "Top"),
                            react_1.default.createElement(Menu_1.MenuItem, { value: Popover_model_1.PopoverAlign.Middle }, "Middle"),
                            react_1.default.createElement(Menu_1.MenuItem, { value: Popover_model_1.PopoverAlign.Bottom }, "Bottom")))),
                    react_1.default.createElement("div", null,
                        react_1.default.createElement("label", { className: Popover_page_styl_1.default.label }, "Horizontal Button Position"),
                        react_1.default.createElement(StatefulOpened, { defaultIsOpened: false, defaultValue: HorizontalButtonPosition.Center, onValueChange: this.onHorizontalAlignSelect },
                            react_1.default.createElement(Menu_1.MenuItem, { value: HorizontalButtonPosition.Left }, "Left"),
                            react_1.default.createElement(Menu_1.MenuItem, { value: HorizontalButtonPosition.Center }, "Center"),
                            react_1.default.createElement(Menu_1.MenuItem, { value: HorizontalButtonPosition.Right }, "Right")),
                        react_1.default.createElement("label", { className: Popover_page_styl_1.default.label }, "Vertical Button Position"),
                        react_1.default.createElement(StatefulOpened, { defaultIsOpened: false, defaultValue: VericalButtonPosition.Middle, onValueChange: this.onVerticalAlignSelect },
                            react_1.default.createElement(Menu_1.MenuItem, { value: VericalButtonPosition.Top }, "Top"),
                            react_1.default.createElement(Menu_1.MenuItem, { value: VericalButtonPosition.Middle }, "Middle"),
                            react_1.default.createElement(Menu_1.MenuItem, { value: VericalButtonPosition.Bottom }, "Bottom")))),
                react_1.default.createElement(Scrollable_1.Scrollable, { theme: scrollableTheme },
                    react_1.default.createElement("div", { ref: function (el) { return (_this.block = el); } },
                        react_1.default.createElement("div", { className: wrapperClassName },
                            react_1.default.createElement(Button_1.Button, { isPrimary: true, onClick: this.onToggle, ref: function (el) { return (_this.anchor = el); } }, "Target"),
                            isOpened && (react_1.default.createElement(Popover_1.Popover, { container: this.block, anchor: this.anchor, isOpened: true, closeOnClickAway: true, disableCloseOnScroll: true, onRequestClose: this.onPopoverRequestClose, hasArrow: true, placement: placement, align: align },
                                react_1.default.createElement("div", null, "popover content")))))))));
    };
    FixedPopoverPage = tslib_1.__decorate([
        pure_1.PURE
    ], FixedPopoverPage);
    return FixedPopoverPage;
}(react_1.default.Component));
exports.FixedPopoverPage = FixedPopoverPage;
//# sourceMappingURL=FixedPopover.page.js.map