"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Option_1 = require("fp-ts/lib/Option");
var Eq_1 = require("fp-ts/lib/Eq");
var eq_utils_1 = require("@devexperts/utils/dist/typeclasses/eq/eq.utils");
var pipeable_1 = require("fp-ts/lib/pipeable");
var PeriodType;
(function (PeriodType) {
    PeriodType["AM"] = "AM";
    PeriodType["PM"] = "PM";
})(PeriodType = exports.PeriodType || (exports.PeriodType = {}));
var Section;
(function (Section) {
    Section[Section["Hours"] = 0] = "Hours";
    Section[Section["Minutes"] = 1] = "Minutes";
    Section[Section["Seconds"] = 2] = "Seconds";
    Section[Section["PeriodType"] = 3] = "PeriodType";
})(Section = exports.Section || (exports.Section = {}));
exports.MAX_VALID_MINS_AND_SEC = 59;
exports.MAX_VALID_HOURS_FOR_24H_FORMAT = 23;
exports.MAX_VALID_HOURS_FOR_12H_FORMAT = 12;
exports.EMPTY_SECTION = '--';
var eqPeriodType = eq_utils_1.eqUnknown;
var eqOptionNumber = Option_1.getEq(Eq_1.eqNumber);
var eqOptionPeriodType = Option_1.getEq(eqPeriodType);
var eqTimeInputValue = Eq_1.getStructEq({
    hours: eqOptionNumber,
    minutes: eqOptionNumber,
    seconds: eqOptionNumber,
    periodType: eqOptionPeriodType,
});
exports.isTimesDifferent = function (x, y) {
    return !eqTimeInputValue.equals(x, y);
};
exports.formatNumericValue = function (value) {
    if (value < 10) {
        return "0" + value;
    }
    return "" + value;
};
exports.formatTimePeriod = function (periodType) {
    switch (periodType) {
        case PeriodType.AM:
            return 'am';
        case PeriodType.PM:
            return 'pm';
    }
};
/**
 * Values can be zeros (start from 0). Max is included value.
 */
function add(a, b, max) {
    return pipeable_1.pipe(a, Option_1.map(function (a) {
        var rawResult = (a + b) % (max + 1);
        return rawResult < 0 ? rawResult + max + 1 : rawResult;
    }), Option_1.alt(function () { return Option_1.some(b < 0 ? max : 0); }));
}
exports.add = add;
function isDefined(value) {
    return typeof value !== 'undefined';
}
exports.isDefined = isDefined;
function findActiveSectionOnKeyLeft(activeState, isSecondsExist) {
    switch (activeState) {
        case Section.PeriodType: {
            return isSecondsExist ? Section.Seconds : Section.Minutes;
        }
        case Section.Seconds: {
            return Section.Minutes;
        }
        case Section.Minutes: {
            return Section.Hours;
        }
        default:
            return Section.Hours;
    }
}
exports.findActiveSectionOnKeyLeft = findActiveSectionOnKeyLeft;
function findActiveSectionOnKeyRight(activeSection, isSecondsExist, isClockFormatExist) {
    switch (activeSection) {
        case Section.Hours: {
            return Section.Minutes;
        }
        case Section.Minutes: {
            if (isSecondsExist) {
                return Section.Seconds;
            }
            else if (isClockFormatExist) {
                return Section.PeriodType;
            }
            else {
                return Section.Minutes;
            }
        }
        case Section.Seconds: {
            return isClockFormatExist ? Section.PeriodType : Section.Seconds;
        }
        case Section.PeriodType: {
            return Section.PeriodType;
        }
        default:
            return Section.Hours;
    }
}
exports.findActiveSectionOnKeyRight = findActiveSectionOnKeyRight;
function togglePeriodType(periodType) {
    var periodTypeNormalized = Option_1.getOrElse(function () { return PeriodType.AM; })(periodType);
    switch (periodTypeNormalized) {
        case PeriodType.AM:
            return Option_1.some(PeriodType.PM);
        case PeriodType.PM:
            return Option_1.some(PeriodType.AM);
    }
}
exports.togglePeriodType = togglePeriodType;
exports.renderSection = Option_1.getOrElse(function () { return exports.EMPTY_SECTION; });
//# sourceMappingURL=TimeInput.model.js.map