"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var react_1 = tslib_1.__importDefault(require("react"));
var react_dom_1 = tslib_1.__importDefault(require("react-dom"));
var classnames_1 = tslib_1.__importDefault(require("classnames"));
var object_1 = require("@devexperts/utils/dist/object/object");
var react_dnd_1 = require("react-dnd");
var react_dnd_html5_backend_1 = require("react-dnd-html5-backend");
var debounce_1 = tslib_1.__importDefault(require("@devexperts/utils/dist/function/debounce"));
var prefix_1 = tslib_1.__importDefault(require("@devexperts/utils/dist/dom/prefix"));
var resize_handler_component_1 = require("./resize-handler.component");
var with_defaults_1 = require("../../utils/with-defaults");
var Pure_1 = require("../Pure/Pure");
var Scrollable_1 = require("../Scrollable/Scrollable");
var pure_1 = require("../../utils/pure");
var ResizeDetector_1 = require("../ResizeDetector/ResizeDetector");
var withTheme_1 = require("../../utils/withTheme");
var layout_constants_1 = require("./layout.constants");
var layout_utils_1 = require("./layout.utils");
var layout_redux_1 = require("./layout.redux");
var pipeable_1 = require("fp-ts/lib/pipeable");
/**
 * @type {boolean}
 */
var resizeHandlerMouseDown = false;
if (typeof document !== 'undefined') {
    document.addEventListener('mouseup', function (e) {
        resizeHandlerMouseDown = false;
    }, true);
}
var RawItemWithNode = /** @class */ (function (_super) {
    tslib_1.__extends(RawItemWithNode, _super);
    function RawItemWithNode() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = {};
        return _this;
    }
    RawItemWithNode.prototype.componentDidMount = function () {
        this.setState({
            node: react_dom_1.default.findDOMNode(this),
        });
    };
    RawItemWithNode.prototype.render = function () {
        var children = this.props.children;
        var node = this.state.node;
        return node ? react_1.default.cloneElement(children, { node: node }) : children;
    };
    return RawItemWithNode;
}(react_1.default.Component));
//do not use @PURE
var RawItem = /** @class */ (function (_super) {
    tslib_1.__extends(RawItem, _super);
    function RawItem() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = {
            draggingItemId: null,
            resizeStartData: null,
            resizeData: null,
        };
        _this._node = null;
        _this.onPlainRemove = function () {
            var _a = _this.props, item = _a.item, removeItem = _a.removeItem;
            removeItem(item.id);
        };
        _this.onNativeDragStart = function (e) {
            if (resizeHandlerMouseDown) {
                e.preventDefault();
                e.stopPropagation();
            }
        };
        _this.onNestedItemDragStart = function (item) {
            _this.setState({
                draggingItemId: item.id,
            });
        };
        _this.onNestedItemDragEnd = function (item) {
            _this.setState({
                draggingItemId: null,
            });
        };
        _this.onNestedItemDrop = function (itemId, destinationId, side) {
            var moveItemTo = _this.props.moveItemTo;
            moveItemTo(itemId, destinationId, side);
        };
        _this.onNativeDrop = function (props, destinationId, side) {
            var _a = _this.props, insertNativeItemAt = _a.insertNativeItemAt, itemPropsFactory = _a.itemPropsFactory;
            insertNativeItemAt(props, destinationId, side);
        };
        _this.onResizeHandlerMouseDown = function () {
            resizeHandlerMouseDown = true;
        };
        _this.onResizeHandlerDragStart = function (previousId, nextId, e, data) {
            if (_this.props.item.type === layout_constants_1.LayoutModelItemType.Layout) {
                var orientation_1 = _this.props.item.orientation;
                var _a = data.node, previousSibling = _a.previousSibling, nextSibling = _a.nextSibling, x = data.x, y = data.y;
                var previousBounds = previousSibling.getBoundingClientRect();
                var nextBounds = nextSibling.getBoundingClientRect();
                var _b = _this.props, minItemWidth = _b.minItemWidth, minItemHeight = _b.minItemHeight, layout = _b.layout, resizeHandlerSize = _b.resizeHandlerSize, item = _b.item;
                var previousItem = layout.items[previousId];
                var previousAmount = layout_utils_1.countChildren(layout, previousItem, orientation_1);
                var nextItem = layout.items[nextId];
                var nextAmount = layout_utils_1.countChildren(layout, nextItem, orientation_1);
                var previousMinParam = orientation_1 === layout_constants_1.LayoutOrientation.Vertical
                    ? layout_utils_1.getMinHeight(previousItem, minItemHeight)
                    : layout_utils_1.getMinWidth(previousItem, minItemWidth);
                var nextMinParam = orientation_1 === layout_constants_1.LayoutOrientation.Vertical
                    ? layout_utils_1.getMinHeight(nextItem, minItemHeight)
                    : layout_utils_1.getMinWidth(nextItem, minItemWidth);
                var minPreviousSize = previousMinParam * previousAmount + (previousAmount - 1) * resizeHandlerSize;
                var minNextSize = nextMinParam * nextAmount + (nextAmount - 1) * resizeHandlerSize;
                var newState = {
                    resizeData: null,
                    resizeStartData: tslib_1.__assign({ previousId: previousId,
                        nextId: nextId,
                        minPreviousSize: minPreviousSize,
                        minNextSize: minNextSize }, layout_utils_1.chooseByOrientation({
                        offset: x,
                        originalPreviousSize: previousBounds.width,
                        originalNextSize: nextBounds.width,
                    }, {
                        offset: y,
                        originalPreviousSize: previousBounds.height,
                        originalNextSize: nextBounds.height,
                    }, orientation_1)),
                };
                _this.setState(newState);
            }
        };
        _this.onResizeHandlerDragStop = function (e, data) {
            var _a = _this.state, resizeStartData = _a.resizeStartData, resizeData = _a.resizeData;
            if (resizeData && resizeStartData && resizeData.offset !== resizeStartData.offset) {
                var _b = _this.props, layout = _b.layout, resizeItems = _b.resizeItems;
                var previousId = resizeStartData.previousId, nextId = resizeStartData.nextId, originalPreviousSize = resizeStartData.originalPreviousSize;
                var items = layout.items;
                var diff = resizeData.offset - resizeStartData.offset;
                var previous = items[previousId];
                var next = items[nextId];
                var k = (originalPreviousSize + diff) / originalPreviousSize;
                var newPreviousSize = Number((previous.size * k).toFixed(5));
                var newNextSize = previous.size + next.size - newPreviousSize;
                resizeItems(previousId, newPreviousSize, nextId, newNextSize);
            }
            _this.setState({
                resizeData: null,
                resizeStartData: null,
            });
        };
        _this.onResizeHandlerDrag = function (e, data) {
            if (_this.props.item.type === layout_constants_1.LayoutModelItemType.Layout) {
                var x = data.x, y = data.y;
                var resizeStartData = _this.state.resizeStartData;
                if (resizeStartData) {
                    var originalPreviousSize = resizeStartData.originalPreviousSize, originalNextSize = resizeStartData.originalNextSize, minPreviousSize = resizeStartData.minPreviousSize, minNextSize = resizeStartData.minNextSize, offset = resizeStartData.offset;
                    var orientation_2 = _this.props.item.orientation;
                    var diff = layout_utils_1.chooseByOrientation(x - offset, y - offset, orientation_2);
                    var newOffset_1 = layout_utils_1.chooseByOrientation(x, y, orientation_2);
                    var newPreviousSize = originalPreviousSize + diff;
                    var newNextSize = originalNextSize - diff;
                    if (newPreviousSize < minPreviousSize) {
                        //dragged to the left too much, take minimum
                        newOffset_1 = offset + minPreviousSize - originalPreviousSize;
                    }
                    else if (newNextSize < minNextSize) {
                        //dragged to the right too much, take minimum
                        newOffset_1 = offset - minNextSize + originalNextSize;
                    }
                    if (typeof newOffset_1 !== 'undefined' && newOffset_1 !== offset) {
                        requestAnimationFrame(function () {
                            _this.setState({
                                resizeData: {
                                    offset: newOffset_1,
                                },
                            });
                        });
                    }
                }
            }
        };
        return _this;
    }
    RawItem.prototype.shouldComponentUpdate = function (newProps, newState) {
        //now we need to remove theme object from original props to avoid checking by shouldComponentUpdate
        var thisPropsCopy = tslib_1.__assign({}, this.props);
        var newPropsCopy = Object.assign({}, newProps);
        delete thisPropsCopy['theme'];
        delete newPropsCopy['theme'];
        delete thisPropsCopy['layout'];
        delete newPropsCopy['layout'];
        delete thisPropsCopy['sizeMap'];
        delete newPropsCopy['sizeMap'];
        //check
        return (
        //either props has changed (ignoring theme and layout which is always new)
        pure_1.shouldComponentUpdate(thisPropsCopy, this.state, newPropsCopy, newState) ||
            //or theme has changed
            !object_1.deepEqual(this.props.theme, newProps.theme));
    };
    RawItem.prototype.componentDidMount = function () {
        var _a = this.props, item = _a.item, layout = _a.layout;
        if (item.type === layout_constants_1.LayoutModelItemType.Plain || layout_utils_1.isEmptyRoot(layout, item)) {
            this._node = react_dom_1.default.findDOMNode(this);
        }
    };
    RawItem.prototype.componentDidUpdate = function () {
        var _a = this.props, item = _a.item, layout = _a.layout;
        if (item.type === layout_constants_1.LayoutModelItemType.Plain || layout_utils_1.isEmptyRoot(layout, item)) {
            this._node = react_dom_1.default.findDOMNode(this);
        }
    };
    RawItem.prototype.componentWillReceiveProps = function (newProps, newState) {
        if (newProps.item !== this.props.item) {
            this.setState({
                draggingItemId: null,
                resizeStartData: null,
                resizeData: null,
            });
        }
    };
    RawItem.prototype.render = function () {
        var _a = this.props, item = _a.item, layout = _a.layout;
        switch (item.type) {
            case layout_constants_1.LayoutModelItemType.Layout: {
                if (layout_utils_1.isEmptyRoot(layout, item)) {
                    return this.renderEmptyRoot();
                }
                else {
                    return this.renderNested(item);
                }
            }
            case layout_constants_1.LayoutModelItemType.Plain: {
                return this.renderPlain(item);
            }
        }
    };
    RawItem.prototype.renderEmptyRoot = function () {
        var _a = this.props, width = _a.width, height = _a.height, theme = _a.theme, isOver = _a.isOver, isFixed = _a.isFixed, connectDropTarget = _a.connectDropTarget;
        var style = {
            width: width,
            height: height,
        };
        var className = classnames_1.default(theme.item, theme.item_nested);
        var child = (react_1.default.createElement("div", { className: className, style: style }, isOver && this.renderPlaceholder()));
        return !isFixed ? connectDropTarget(child) : child;
    };
    RawItem.prototype.renderNested = function (item) {
        var _a;
        var _this = this;
        var _b = this.props, layout = _b.layout, itemPropsFactory = _b.itemPropsFactory, width = _b.width, height = _b.height, x = _b.x, y = _b.y, theme = _b.theme, resizeItems = _b.resizeItems, moveItemTo = _b.moveItemTo, insertNativeItemAt = _b.insertNativeItemAt, insertNativeItemAtEmptyRoot = _b.insertNativeItemAtEmptyRoot, removeItem = _b.removeItem, ItemComponent = _b.ItemComponent, resizeHandlerSize = _b.resizeHandlerSize, minItemWidth = _b.minItemWidth, minItemHeight = _b.minItemHeight, Placeholder = _b.Placeholder, isFixed = _b.isFixed, maximizedId = _b.maximizedId;
        var sizeMap = this.props.sizeMap;
        var item_ids = item.item_ids, orientation = item.orientation;
        var items = item_ids.map(function (id) { return layout.items[id]; });
        var style = tslib_1.__assign({ width: width,
            height: height }, prefix_1.default({
            transform: "translate(" + x + "px, " + y + "px)",
        }));
        var _c = this.state, draggingItemId = _c.draggingItemId, resizeStartData = _c.resizeStartData, resizeData = _c.resizeData;
        var isResizing = resizeStartData && resizeData;
        if (draggingItemId) {
            //ignore current dragging item in size recalculation
            sizeMap = layout_utils_1.getSizeMap(layout, item, minItemWidth, minItemHeight, width, height, resizeHandlerSize, 0, draggingItemId, tslib_1.__assign({}, sizeMap));
        }
        else if (resizeStartData && resizeData) {
            //is resizing - update sizeMap
            var diff = resizeData.offset - resizeStartData.offset;
            var previousSize = sizeMap[resizeStartData.previousId];
            var nextSize = sizeMap[resizeStartData.nextId];
            //make a copy
            sizeMap = tslib_1.__assign({}, sizeMap);
            switch (orientation) {
                case layout_constants_1.LayoutOrientation.Vertical: {
                    sizeMap = layout_utils_1.getSizeMap(layout, layout.items[resizeStartData.previousId], minItemWidth, minItemHeight, previousSize.width, previousSize.height + diff, resizeHandlerSize, 0, '', sizeMap);
                    sizeMap = layout_utils_1.getSizeMap(layout, layout.items[resizeStartData.nextId], minItemWidth, minItemHeight, nextSize.width, nextSize.height - diff, resizeHandlerSize, 0, '', sizeMap);
                    break;
                }
                case layout_constants_1.LayoutOrientation.Horizontal: {
                    sizeMap = layout_utils_1.getSizeMap(layout, layout.items[resizeStartData.previousId], minItemWidth, minItemHeight, previousSize.width + diff, previousSize.height, resizeHandlerSize, 0, '', sizeMap);
                    sizeMap = layout_utils_1.getSizeMap(layout, layout.items[resizeStartData.nextId], minItemWidth, minItemHeight, nextSize.width - diff, nextSize.height, resizeHandlerSize, 0, '', sizeMap);
                    break;
                }
            }
        }
        var className = classnames_1.default(theme.item, theme.item_nested, (_a = {},
            _a[theme.item_horizontal] = orientation === layout_constants_1.LayoutOrientation.Horizontal,
            _a[theme.item_vertical] = orientation === layout_constants_1.LayoutOrientation.Vertical,
            _a));
        var sizes = layout_utils_1.chooseByOrientation(function () { return item_ids.map(function (id) { return sizeMap[id].width; }); }, function () { return item_ids.map(function (id) { return sizeMap[id].height; }); }, orientation)();
        var offset = 0;
        return (react_1.default.createElement("div", { className: className, style: style }, items.reduce(function (acc, nestedItem, i) {
            var itemSize = sizeMap[nestedItem.id];
            var size = layout_utils_1.chooseByOrientation(itemSize.width, itemSize.height, orientation);
            var isResizingThisItem = isResizing &&
                resizeStartData &&
                (nestedItem.id === resizeStartData.previousId || nestedItem.id === resizeStartData.nextId);
            var itemWidth = layout_utils_1.chooseByOrientation(size, itemSize.width, orientation);
            var itemHeight = layout_utils_1.chooseByOrientation(itemSize.height, size, orientation);
            //eslint-disable-next-line
            var itemX = layout_utils_1.chooseByOrientation(offset, void 0, orientation);
            //eslint-disable-next-line
            var itemY = layout_utils_1.chooseByOrientation(void 0, offset, orientation);
            offset += layout_utils_1.chooseByOrientation(itemWidth, itemHeight, orientation);
            acc.push(react_1.default.createElement(RawItemWithNode, { key: nestedItem.id },
                react_1.default.createElement(NestedItem, { sizeMap: sizeMap, theme: theme, item: nestedItem, width: itemWidth, height: itemHeight, x: itemX, y: itemY, isResizing: Boolean(isResizingThisItem), resizeHandlerSize: resizeHandlerSize, minItemWidth: minItemWidth, minItemHeight: minItemHeight, onDragStart: _this.onNestedItemDragStart, onDragEnd: _this.onNestedItemDragEnd, onDrop: _this.onNestedItemDrop, onNativeDrop: _this.onNativeDrop, resizeItems: resizeItems, moveItemTo: moveItemTo, insertNativeItemAt: insertNativeItemAt, insertNativeItemAtEmptyRoot: insertNativeItemAtEmptyRoot, removeItem: removeItem, ItemComponent: ItemComponent, Placeholder: Placeholder, itemPropsFactory: itemPropsFactory, maximizedId: maximizedId, isFixed: isFixed, layout: layout })));
            //add resize handler if needed
            var resizeHandlerSide = layout_utils_1.getResizeHandlerSide(sizes, i);
            if (resizeHandlerSide === layout_constants_1.ResizeHandlerSide.Both || resizeHandlerSide === layout_constants_1.ResizeHandlerSide.After) {
                acc.push(react_1.default.createElement(resize_handler_component_1.ResizeHandler, { key: "resizeHandler_" + i, onDrag: _this.onResizeHandlerDrag, onDragStart: _this.onResizeHandlerDragStart, onDragStop: _this.onResizeHandlerDragStop, onMouseDown: _this.onResizeHandlerMouseDown, previousId: nestedItem.id, nextId: items[i + 1].id, theme: theme, offset: offset, orientation: orientation, isFixed: isFixed, size: resizeHandlerSize }));
                offset += resizeHandlerSize;
            }
            return acc;
        }, [])));
    };
    RawItem.prototype.renderPlain = function (item) {
        var _this = this;
        var _a = this.props, theme = _a.theme, width = _a.width, height = _a.height, isFixed = _a.isFixed, connectDropTarget = _a.connectDropTarget, connectDragSource = _a.connectDragSource, connectDragPreview = _a.connectDragPreview, ItemComponent = _a.ItemComponent, isOver = _a.isOver, x = _a.x, y = _a.y;
        var style = tslib_1.__assign({ width: width,
            height: height }, prefix_1.default({
            transform: "translate(" + x + "px, " + y + "px)",
        }));
        var props = tslib_1.__assign(tslib_1.__assign({}, item.props), { connectDragSource: connectDragSource });
        var child = (react_1.default.createElement("div", { className: theme.item, style: style, onDragStart: this.onNativeDragStart },
            react_1.default.createElement(Pure_1.Pure, { ItemComponent: ItemComponent, props: props, connectDragSource: connectDragSource }, function () { return react_1.default.createElement(ItemComponent, tslib_1.__assign({ onRemove: _this.onPlainRemove }, props)); }),
            isOver && this.renderPlaceholder()));
        return !isFixed ? connectDropTarget(connectDragPreview(child)) : child;
    };
    RawItem.prototype.renderPlaceholder = function () {
        var _a;
        var _b = this.props, theme = _b.theme, layout = _b.layout, item = _b.item, Placeholder = _b.Placeholder, maximizedId = _b.maximizedId;
        var isMaximized = maximizedId === item.id;
        var className = theme.placeholder;
        var _c = this.props, hoverSide = _c.hoverSide, hoverFits = _c.hoverFits;
        var emptyRoot = item.type === layout_constants_1.LayoutModelItemType.Layout && layout_utils_1.isEmptyRoot(layout, item);
        var fits = emptyRoot || hoverFits;
        var full = emptyRoot;
        if (this._node) {
            if (typeof hoverSide === 'undefined') {
                return null;
            }
            className = classnames_1.default(className, (_a = {},
                _a[theme.placeholder_fits] = fits,
                _a[theme.placeholder_bottom] = hoverSide === layout_constants_1.ItemSide.Bottom,
                _a[theme.placeholder_top] = hoverSide === layout_constants_1.ItemSide.Top,
                _a[theme.placeholder_left] = hoverSide === layout_constants_1.ItemSide.Left,
                _a[theme.placeholder_right] = hoverSide === layout_constants_1.ItemSide.Right,
                _a[theme.placeholder_maximized] = isMaximized,
                _a[theme.placeholder_full] = full || isMaximized,
                _a));
        }
        return (react_1.default.createElement(Placeholder, { className: className, side: hoverSide, fits: fits, isFull: full, isMaximized: isMaximized }));
    };
    RawItem.defaultProps = {
        x: 0,
        y: 0,
    };
    return RawItem;
}(react_1.default.Component));
var itemSource = {
    beginDrag: function (props) {
        if (/firefox/i.test(navigator.userAgent)) {
            props.onDragStart(props.item);
        }
        else {
            setTimeout(function () {
                //setTimeout here because new sizes come to LayoutItems before returning null from dragging
                //if this happens then drag source size is decreased because of increasing other sizes in percents
                //if it is decreased too much then browser thinks that the item is not being dragged because pointer is now
                //out of previous draggable bounds
                props.onDragStart(props.item);
            }, 0);
        }
        return {
            id: props.item.id,
        };
    },
    endDrag: function (props, monitor) {
        if (monitor.didDrop()) {
            var result = monitor.getDropResult();
            if (typeof result.side !== 'undefined') {
                //this is for plain items only
                //onRemove for nested layouts is called in LayoutItem
                props.onDrop && props.onDrop(props.item.id, result.item.id, result.side);
            }
            else {
                props.onDragEnd(props.item);
            }
        }
        else {
            props.onDragEnd(props.item);
        }
        return {
            id: props.item.id,
        };
    },
    canDrag: function (props) {
        var layout = props.layout, isFixed = props.isFixed, maximizedId = props.maximizedId, item = props.item;
        var root_id = layout.root_id, items = layout.items;
        var layoutItem = items[root_id];
        var amount = layout_utils_1.countChildren(layout, layoutItem, layoutItem.orientation);
        var isEnabledDrag = amount > 1 || (amount === 1 && Object.keys(items).length > 2);
        return !isFixed && !props.isResizing && isEnabledDrag && maximizedId !== item.id;
    },
};
var itemTarget = {
    hover: function (props, monitor, component) {
        var canDropResult = monitor.canDrop();
        if (canDropResult && canDropResult.offset && canDropResult.placement) {
            var _a = canDropResult.placement, fits = _a.fits, side = _a.side;
            component.setState({
                hoverSide: side,
                hoverFits: fits,
            });
        }
    },
    drop: function (props, monitor, component) {
        var isNativeDropAtEmptyRoot = props.item.type === layout_constants_1.LayoutModelItemType.Layout && layout_utils_1.isEmptyRoot(props.layout, props.item);
        var domNode = react_dom_1.default.findDOMNode(component);
        if (domNode === null) {
            return;
        }
        var offset = monitor.getClientOffset();
        var bounds = domNode.getBoundingClientRect();
        //noinspection JSSuspiciousNameCombination
        var placement = layout_utils_1.getPlaceholderPlacement(Math.floor(bounds.left), Math.floor(bounds.top), Math.floor(bounds.right), Math.floor(bounds.bottom), offset.x, offset.y, layout_utils_1.getMinWidth(props.item, props.minItemWidth), layout_utils_1.getMinHeight(props.item, props.minItemHeight), props.maximizedId === props.item.id);
        if (typeof placement.side !== 'undefined' && placement.fits) {
            if (monitor.getItemType() === react_dnd_html5_backend_1.NativeTypes.TEXT) {
                var data = undefined;
                try {
                    data = JSON.parse(monitor.getItem()['text']);
                }
                catch (e) { }
                if (data) {
                    if (typeof data[layout_constants_1.NATIVE_DRAG_DATA_KEY] !== 'undefined') {
                        var itemProps = props.itemPropsFactory(data[layout_constants_1.NATIVE_DRAG_DATA_KEY]);
                        if (isNativeDropAtEmptyRoot) {
                            //native drop to empty root
                            props.insertNativeItemAtEmptyRoot(itemProps);
                        }
                        else {
                            //native drop of plain item
                            props.onNativeDrop(itemProps, props.item.id, placement.side);
                        }
                    }
                    /* else if (typeof data.item !== 'undefined' && data[SESSION_ID_KEY] !== session.id) {
                     //drop of item from another tab
                     props.onNativeDrop(data.item, props.item.id, side);
                     //try to notify source tab about drop - cannot handle =(
                     // session.send(data[SESSION_ID_KEY], E_LAYOUT.ITEM_DROP, {
                     // 	item: data.item
                     // });

                     //data.item.id is not set here
                     // props.onDrop && props.onDrop(data.item.id, props.item.id, side);
                     }*/
                }
            }
        }
        //this return value is accessed in itemSource.endDrag callback
        return {
            side: placement.fits ? placement.side : undefined,
            item: props.item,
        };
    },
    canDrop: function (props, monitor) {
        var emptyRoot = props.item.type === layout_constants_1.LayoutModelItemType.Layout && layout_utils_1.isEmptyRoot(props.layout, props.item);
        if (props.item.type === layout_constants_1.LayoutModelItemType.Plain || emptyRoot) {
            var offset = monitor.getClientOffset();
            if (offset) {
                var placement = void 0;
                if (props.node) {
                    var bounds = props.node.getBoundingClientRect();
                    placement = layout_utils_1.getPlaceholderPlacement(Math.floor(bounds.left), Math.floor(bounds.top), Math.floor(bounds.right), Math.floor(bounds.bottom), offset.x, offset.y, layout_utils_1.getMinWidth(props.item, props.minItemWidth), layout_utils_1.getMinHeight(props.item, props.minItemHeight), props.maximizedId === props.item.id);
                }
                return {
                    offset: offset,
                    placement: placement,
                    isEmptyRoot: emptyRoot,
                };
            }
        }
        return false;
    },
};
var dndKey = 'DRAG_DROP.ITEM';
var Item = pipeable_1.pipe(RawItem, react_dnd_1.DragSource(dndKey, itemSource, function (connect, monitor) { return ({
    connectDragSource: connect.dragSource(),
    connectDragPreview: connect.dragPreview(),
}); }), react_dnd_1.DropTarget([dndKey, react_dnd_html5_backend_1.NativeTypes.TEXT], itemTarget, function (connect, monitor) { return ({
    connectDropTarget: connect.dropTarget(),
    isOver: monitor.isOver({
        shallow: true,
    }),
}); }));
var NestedItem = Item;
var RootItem = Item;
var layoutDefaults = with_defaults_1.withDefaults({
    isFixed: false,
    Scrollable: Scrollable_1.Scrollable,
    Placeholder: function (props) { return react_1.default.createElement("div", { className: props.className }); },
});
var RawLayout = /** @class */ (function (_super) {
    tslib_1.__extends(RawLayout, _super);
    function RawLayout() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = {};
        _this.renderRootItem = function (width, height, maximizedId) {
            var _a;
            var _b = _this.props, theme = _b.theme, resizeHandlerSize = _b.resizeHandlerSize, minItemWidth = _b.minItemWidth, minItemHeight = _b.minItemHeight, ItemComponent = _b.ItemComponent, itemPropsFactory = _b.itemPropsFactory, Placeholder = _b.Placeholder, isFixed = _b.isFixed;
            var value = _this.props.value;
            if (maximizedId) {
                var rootItem = value.items[value.root_id];
                if (rootItem.type === layout_constants_1.LayoutModelItemType.Plain) {
                    return;
                }
                value = {
                    root_id: value.root_id,
                    items: (_a = {},
                        _a[value.root_id] = tslib_1.__assign(tslib_1.__assign({}, rootItem), { item_ids: [maximizedId] }),
                        _a[maximizedId] = value.items[maximizedId],
                        _a),
                };
            }
            var root = value.items[value.root_id];
            var sizeMap = layout_utils_1.getSizeMap(value, value.items[value.root_id], minItemWidth, minItemHeight, width, height, resizeHandlerSize, 0);
            return (react_1.default.createElement(RootItem, { item: root, Placeholder: Placeholder, sizeMap: sizeMap, ItemComponent: ItemComponent, itemPropsFactory: itemPropsFactory, layout: value, width: sizeMap[root.id].width, height: sizeMap[root.id].height, resizeHandlerSize: resizeHandlerSize, minItemWidth: minItemWidth, minItemHeight: minItemHeight, theme: theme, resizeItems: _this.resizeItems, insertNativeItemAt: _this.insertNativeItemAt, insertNativeItemAtEmptyRoot: _this.insertNativeItemAtEmptyRoot, removeItem: _this.removeItem, moveItemTo: _this.moveItemTo, maximizedId: maximizedId, isFixed: isFixed }));
        };
        _this.moveItemTo = function (id, destinationId, side) {
            _this.dispatch(layout_redux_1.moveTo(id, destinationId, side));
        };
        _this.resizeItems = function (id1, size1, id2, size2) {
            _this.dispatch(layout_redux_1.resize(id1, size1, id2, size2));
        };
        _this.insertNativeItemAt = function (props, id, side) {
            _this.dispatch(layout_redux_1.insertNativeAt(props, id, side));
        };
        _this.insertNativeItemAtEmptyRoot = function (props) {
            _this.dispatch(layout_redux_1.insertNativeAtEmptyRoot(props));
        };
        _this.removeItem = function (id) {
            _this.dispatch(layout_redux_1.remove(id));
        };
        _this.dispatch = function (action) {
            _this.props.onChange && _this.props.onChange(layout_redux_1.reducer(_this.props.value, action));
        };
        _this.handleResize = function () {
            _this.updateSize();
        };
        return _this;
    }
    RawLayout.prototype.componentWillMount = function () {
        var debounceResize = this.props.debounceResize;
        this.onResize = debounceResize ? debounce_1.default(this.handleResize, debounceResize) : this.handleResize;
    };
    RawLayout.prototype.componentWillUpdate = function (newProps) {
        var debounceResize = newProps.debounceResize;
        if (debounceResize !== this.props.debounceResize) {
            this.onResize = debounceResize ? debounce_1.default(this.handleResize, debounceResize) : this.handleResize;
        }
    };
    RawLayout.prototype.componentDidMount = function () {
        this.updateSize();
    };
    RawLayout.prototype.render = function () {
        var _a = this.props, theme = _a.theme, Scrollable = _a.Scrollable;
        var _b = this.state, width = _b.width, height = _b.height;
        var style = {
            width: width,
            height: height,
        };
        return (react_1.default.createElement("div", { className: theme.container },
            width && height && (react_1.default.createElement("div", { style: style, className: theme.fixedContainer },
                react_1.default.createElement(Scrollable, { theme: theme.Scrollable }, this.renderRoot(width, height)))),
            react_1.default.createElement(ResizeDetector_1.ResizeDetector, { onResize: this.onResize })));
    };
    RawLayout.prototype.renderRoot = function (width, height) {
        var maximizedId = this.props.maximizedId;
        return react_1.default.createElement(RawItemWithNode, null, this.renderRootItem(width, height, maximizedId));
    };
    RawLayout.prototype.updateSize = function () {
        var element = react_dom_1.default.findDOMNode(this);
        var offsetWidth = element.offsetWidth, offsetHeight = element.offsetHeight;
        this.setState({
            width: offsetWidth,
            height: offsetHeight,
        });
    };
    RawLayout = tslib_1.__decorate([
        pure_1.PURE
    ], RawLayout);
    return RawLayout;
}(react_1.default.Component));
exports.Layout = withTheme_1.withTheme(layout_constants_1.LAYOUT)(layoutDefaults(RawLayout));
//# sourceMappingURL=layout.component.js.map